import React, { useEffect } from 'react'
import Cookies from 'cookies-js'

export default function CoachMark (props) {
  useEffect(() => {
    init()
    return () => {
      destroy()
    }
  })

  let steps = null
  let tooltip = null
  let nextBtn = null
  let skipBtn = null
  let okBtn = null
  let currentStep = 0
  let highlighter = null
  let tooltipTextElement = null
  let tooltipTitleElement = null
  const styleClass = 'coachmark-styles'

  const init = () => {
    const coachmarkSelectors = document.querySelectorAll('*[data-coachmark]')
    const coachmarks = Array.from(coachmarkSelectors) || []
    steps = sortArr(coachmarks)

    const coachmarkElements = getCoachmarkElements()

    document.body.insertAdjacentHTML('beforeend', coachmarkElements)

    highlighter = document.getElementById('js-coachmark-interface')

    tooltip = document.getElementById('js-coachmark-tooltip')

    tooltipTextElement = document.getElementById('js-coachmark-tooltip-text')

    tooltipTitleElement = document.getElementById('js-coachmark-tooltip-title')

    skipBtn = document.querySelector('.coachmark-skip')

    nextBtn = document.querySelector('.coachmark-next')

    okBtn = document.getElementById('ok-btn')

    highlightElement(steps[currentStep])

    skipBtn.addEventListener('click', handleSkipClick)

    okBtn.addEventListener('click', handleOkButtonClick)
  }

  const highlightElement = (element) => {
    if (!element) {
      throw new Error(

        'add data-coachmark attribute.'

      )
    }

    const { top, left } = element.getBoundingClientRect()

    const height = element.offsetHeight

    const width = element.offsetWidth

    const backgroundColor = getBgColor(element)

    const tooltipText = element.dataset.coachmarkText || ''

    const tooltipTitle = element.dataset.coachmarkTitle || ''

    const yScrollPosition = window.pageYOffset

    const xScrollPosition = window.pageXOffset

    if (highlighter) {
      highlighter.style['background-color'] = backgroundColor

      highlighter.style.top = `${top + yScrollPosition}px`

      highlighter.style.left = `${left + xScrollPosition}px`

      highlighter.style.width = `${width}px`

      highlighter.style.height = `${height}px`
    }

    addTooltip({

      height,

      text: tooltipText,

      title: tooltipTitle,

      top: top + yScrollPosition,

      left: left + xScrollPosition

    })

    // Add class to the element

    element.classList.add('coachmark-highlight')

    // Scroll element's parent container so that it is visible

    setTimeout(() => {
      element.scrollIntoView({ behavior: 'smooth', block: 'center' })
    }, 0)

    if (currentStep === steps.length - 1) {
      nextBtn.style.display = 'none'

      skipBtn.style.display = 'none'

      okBtn.style.display = 'inline-block'
    } else {
      nextBtn.style.display = 'inline-block'

      okBtn.style.display = 'none'

      nextBtn.addEventListener('click', handleNextBtnClick)
    }
  }

  const removeHighlight = (element) => {
    if (element) {
      element.classList.remove('coachmark-highlight')
    }
  }

  const addTooltip = ({ top, left, height, text, title }) => {
    if (text === '') console.warn('Please add text for coachmark tooltip')

    tooltip.style.top = `${Number.parseInt(top, 10) + Number.parseInt(height, 10)

      }px`

    tooltip.style.left = `${left}px`

    tooltipTextElement.textContent = text

    tooltipTitleElement.textContent = title
  }

  const handleSkipClick = () => {
    destroy()
  }

  const handleOkButtonClick = () => {
    setCookies()

    destroy()
  }

  const handleNextBtnClick = () => {
    if (currentStep < steps.length - 1) {
      removeHighlight(steps[currentStep])

      currentStep++

      highlightElement(steps[currentStep])
    }
  }

  const setCookies = () => {
    Cookies.set(`coachmark-${props.cookiesName}-${(props.id ? props.id : '')}`, 1, {

      expires: 1095

    })
  }

  const destroy = () => {
    removeHighlight(steps[currentStep])

    const mountPoint = document.getElementById('js-coachmark')

    if (mountPoint) mountPoint.remove()

    if (nextBtn) nextBtn.removeEventListener('click', handleNextBtnClick)

    if (skipBtn) skipBtn.removeEventListener('click', handleSkipClick)

    if (okBtn) okBtn.removeEventListener('click', handleOkButtonClick)

    // Destroy all dynamically inserted styles

    const allDynamicStyles = document.getElementsByClassName(styleClass)

    if (allDynamicStyles.length) {
      Array.from(allDynamicStyles).forEach((dynamicStyle) =>

        dynamicStyle.remove()

      )
    }
  }

  const getCoachmarkElements = () => {
    return `

      <section id="js-coachmark">

      <section id="js-coachmark-overlay" class="coachmark-overlay"></section>

      <section id="js-coachmark-interface" class="coachmark-interface"></section>

        <section id="js-coachmark-tooltip" class="coachmark-tooltip">

          <h5 id="js-coachmark-tooltip-title">Ini judul coachmark</h5>

          <hr>

          <header id="js-coachmark-tooltip-text">

          Please type any text

          </header>

          <article class="coachmark-btns">

              <button class="coachmark-skip __btn-skip">Skip</button>

              <button id="ok-btn" class="coachmark-ok __btn-ok">OK</button>

              <button class="coachmark-next __btn">Next</button>

          </article>

        </section>

      </section>

    `
  }

  const getBgColor = (element) => {
    let backgroundColor = window

      .getComputedStyle(element)

      .getPropertyValue('background-color')

    while (

      backgroundColor === 'rgba(0, 0, 0, 0)' &&

      element !== document.body

    ) {
      element = element.parentNode

      backgroundColor = window

        .getComputedStyle(element)

        .getPropertyValue('background-color')
    }

    return backgroundColor
  }

  const sortArr = (arr) => {
    if (!Array.isArray(arr) || arr.length === 0) return []

    arr.sort((a, b) => {
      const aValue = Number.parseInt(a.dataset.coachmark, 10)

      const bValue = Number.parseInt(b.dataset.coachmark, 10)

      if (aValue < bValue) {
        return -1
      }

      if (aValue > bValue) {
        return 1
      }

      return 0
    })

    return [...arr]
  }

  return <div />
}
